package com.hzya.frame.plugin.lets.dingtalk.utils;

import cn.hutool.core.lang.Assert;
import com.aliyun.dingtalkstorage_1_0.models.AddPermissionResponse;
import com.aliyun.dingtalkstorage_1_0.models.CommitFileRequest;
import com.aliyun.dingtalkstorage_1_0.models.CommitFileResponse;
import com.aliyun.dingtalkstorage_1_0.models.GetFileUploadInfoResponse;
import com.aliyun.dingtalkworkflow_1_0.models.GrantProcessInstanceForDownloadFileResponse;
import com.hzya.frame.plugin.lets.dingtalk.vo.FileModuleVO;
import com.hzya.frame.plugin.lets.dingtalk.vo.SysFileVO;
import org.apache.commons.lang.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Base64;
import java.util.List;
import java.util.Map;

@Component
public class FileUtil {
    protected static Logger logger = LogManager.getLogger(FileUtil.class);

    @Value("${DING.SPACEID}")
    private static String spaceId;

    /**
     * 审批文件上传钉盘
     * @param accessToken 应用Token
     * @param unionid 钉钉用户unionid
     * @param spaceId 钉盘审批实例中附件地址
     * @param fileVO 的文件信息
     * @param stream 文件读取的字节流
     * @param dingFileUrlID 钉盘审批实例中附件地址
     * @return
     */
    public static FileModuleVO get4DingDileModuleVO(String accessToken, String unionid, String spaceId, SysFileVO fileVO, ByteArrayOutputStream stream,String dingFileUrlID){
        FileModuleVO fileModuleVO = new FileModuleVO();
        try {
            //钉钉--上传附件
            //1.调用服务端API-获取审批钉盘空间信息接口，获取审批空间spaceId。
            System.out.println(spaceId);
            //2.调用服务端API-获取文件上传信息接口，获取审批空间文件上传信息并执行上传。
            /////////////////////////////////////////////////////////////////////////////////////////////////此处很容易调权限，钉钉问题。添加权限
            AddPermissionResponse addPermissionResponse = DingTalkUtils.addFileStorageByUnionId(accessToken, spaceId, unionid);
            if(addPermissionResponse.getBody().getSuccess()!=true){
                Assert.state(false,"钉钉添加审批实例附件到钉盘权限失败");
            }
            /////////////////////////////////////////////////////////////////////////////////////////////////此处很容易调权限，钉钉问题。添加权限
            //2.1调用本接口，获取上传文件需要的resourceUrls和headers参数值。
            GetFileUploadInfoResponse headerSignature = DingTalkUtils.getFileUploadInfo(accessToken, spaceId, unionid, "HEADER_SIGNATURE", false);
            String uploadKey = headerSignature.getBody().getUploadKey();
            List<String> resourceUrls = headerSignature.getBody().getHeaderSignatureInfo().getResourceUrls();
            if (resourceUrls.size() > 1) {
                Assert.state(false, "获取文件上传信息接口中resourceUrls不唯一！！！");
            }
            Map<String, String> headers = headerSignature.getBody().getHeaderSignatureInfo().getHeaders();
            //2.2使用OSS的header加签方式上传文件，参考示例：
            // 从接口返回信息中拿到headers
            URL url = new URL(resourceUrls.get(0));
            HttpURLConnection connection = (HttpURLConnection) url.openConnection();
            if (headers != null) {
                for (Map.Entry<String, String> entry : headers.entrySet()) {
                    connection.setRequestProperty(entry.getKey(), entry.getValue());
                }
            }
            connection.setDoOutput(true);
            connection.setRequestMethod("PUT");
            connection.setUseCaches(false);
            connection.setReadTimeout(10000);
            connection.setConnectTimeout(10000);
            connection.connect();
            OutputStream out = connection.getOutputStream();
            out.write(stream.toByteArray());
            out.flush();
            out.close();
            int responseCode = connection.getResponseCode();
            connection.disconnect();
            if (responseCode == 200) {
                logger.info("审批实例附件上传审批钉盘成功,UploadKey:-->" + headerSignature.getBody().getUploadKey());
            } else {
                Assert.state(false,"审批实例附件上传审批钉盘失败");
            }
            //2.3调用提交文件接口，完成文件上传。
            CommitFileRequest.CommitFileRequestOptionAppProperties commitFileRequestOptionAppProperties = new CommitFileRequest.CommitFileRequestOptionAppProperties();
            commitFileRequestOptionAppProperties.setName(fileVO.getFileName());
            commitFileRequestOptionAppProperties.setValue(fileVO.getFileName());
            commitFileRequestOptionAppProperties.setVisibility("PUBLIC");//属性可见性。 - PUBLIC：所有应用都可见 - PRIVATE：仅限当前应用可见

            //3.提交文件
            CommitFileResponse commitFileResponse = DingTalkUtils.submitFile(accessToken, spaceId, unionid, uploadKey, fileVO.getFileName(), dingFileUrlID, commitFileRequestOptionAppProperties);

            //钉盘文件唯一id
            String dingFileUniqueId = commitFileResponse.getBody().getDentry().getId();

            //4.组装钉钉文件控件-->"[{\"spaceId\": \"163xxxx658\", \"fileName\": \"2644.JPG\", \"fileSize\": \"333\", \"fileType\": \"jpg\", \"fileId\": \"643xxxx140\"}]"
            fileModuleVO.setSpaceId(spaceId);
            fileModuleVO.setFileName(fileVO.getFileName());
            fileModuleVO.setFileSize(fileVO.getFileSize());
            fileModuleVO.setFileType(fileVO.getType());//取后缀
            fileModuleVO.setFileId(dingFileUniqueId);

        }catch (Exception e){
            e.printStackTrace();
            logger.error(e);
            Assert.state(false,"文件存钉盘失败,失败信息:{}",e.getMessage());
        }
        return fileModuleVO;
    }

    /**
     * 钉钉审批实例附件下载，转base64编码
     * @Param fileModuleVOList 钉钉附件列表
     * @Param token token
     * @Param processInstanceId 审批实例id
     */
    public List<SysFileVO> DingFileTransBase64(List<FileModuleVO> fileModuleVOList,String token,String processInstanceId) throws Exception {
        List<SysFileVO> sysFileVOList=new ArrayList<>();

        for (FileModuleVO fileModuleVO : fileModuleVOList) {
            SysFileVO sysFileVO = new SysFileVO();
            String targetPath = "D:\\ding-file\\download\\"+processInstanceId+"_"+fileModuleVO.getFileId()+"\\";

            //附件下载
            GrantProcessInstanceForDownloadFileResponse grantProcessInstanceForDownloadFileResponse = DingTalkUtils.downloadProcessFile(token, processInstanceId, fileModuleVO.getFileId());
            String uri = grantProcessInstanceForDownloadFileResponse.getBody().getResult().getDownloadUri();
            URL url = new URL(uri);
            HttpURLConnection httpConn = (HttpURLConnection) url.openConnection();
            int responseCode = httpConn.getResponseCode();
            if (responseCode == HttpURLConnection.HTTP_OK) {
                InputStream inputStream = httpConn.getInputStream();
                FileOutputStream outputStream = new FileOutputStream(targetPath);
                byte[] buffer = new byte[4096];
                int bytesRead;

                while ((bytesRead = inputStream.read(buffer)) != -1) {
                    outputStream.write(buffer, 0, bytesRead);
                }
                //转为base64编码
                // 读取文件内容到字节数组
                byte[] fileContent = Files.readAllBytes(Paths.get(targetPath));
                // 将字节数组转换为 Base64 编码的字符串
                String encodedString = Base64.getEncoder().encodeToString(fileContent);

                sysFileVO.setFileName(fileModuleVO.getFileName());
                sysFileVO.setType(fileModuleVO.getFileType());
                sysFileVO.setBaseCode(encodedString);
                System.out.println(sysFileVO);
                sysFileVOList.add(sysFileVO);

                //关闭资源
                outputStream.close();
                inputStream.close();
                httpConn.disconnect();
            } else {
                throw new IOException("无法下载文件，HTTP响应码: " + responseCode);
            }
        }
        return sysFileVOList;
    }


}
