package com.hzya.frame.plugin.lets.plugin.base;

import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.hzya.frame.base.PluginBaseEntity;
import com.hzya.frame.plugin.lets.constant.OverallConstant;
import com.hzya.frame.plugin.lets.constant.ProfilesActiveConstant;


import com.hzya.frame.plugin.lets.entity.BdCubasdocEntity;
import com.hzya.frame.plugin.lets.queryvo.BankVo;
import com.hzya.frame.plugin.lets.queryvo.CustdocRequestVo;

import com.hzya.frame.plugin.lets.resultvo.CbdocResultVo;
import com.hzya.frame.plugin.lets.u8cdto.ArapDjzb;
import com.hzya.frame.plugin.lets.u8cdto.CustdocEntityDto;
import com.hzya.frame.plugin.lets.util.PushDataByU8cUtil;
import com.hzya.frame.plugin.lets.util.QueryU8CEntityUtil;
import com.hzya.frame.plugin.lets.util.SaveOrUpdateBusinessLogUtil;
import com.hzya.frame.plugin.lets.util.pushData.Attribute;
import com.hzya.frame.plugin.lets.util.pushData.PushU8CByApiCode;
import com.hzya.frame.plugin.lets.util.pushData.ZTResult;
import com.hzya.frame.sysnew.integtationTaskLivingDetails.dao.IIntegrationTaskLivingDetailsDao;
import com.hzya.frame.sysnew.integtationTaskLivingDetails.entity.IntegrationTaskLivingDetailsEntity;
import com.hzya.frame.ttxofs.dto.InterfaceParamByU8CApiDto;

import com.hzya.frame.ttxofs.dto.ofsBaseSearch.OfsSupplierDto;
import com.hzya.frame.ttxofs.dto.ofsBaseSearch.OfsSupplierReturnMessageDto;
import com.hzya.frame.ttxofs.service.OfsUnifiedService;
import com.hzya.frame.web.entity.BaseResult;
import com.hzya.frame.web.entity.JsonResultEntity;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.locks.ReentrantLock;
import java.util.stream.Collectors;

/**
 * @Author
 * @Date 2024/8/6 14:36
 * @PackageName:com.hzya.frame.plugin.lets.plugin.base
 * @ClassName: ShopPluginInitializer
 * @Description: 供应商档案->客商档案
 */
public class SupplierPluginInitializer extends PluginBaseEntity {
    Logger logger = LoggerFactory.getLogger(SupplierPluginInitializer.class);

    private static final ReentrantLock LOCK = new ReentrantLock(true);

    @Autowired
    private OfsUnifiedService ofsUnifiedService;
    @Autowired
    private PushU8CByApiCode pushU8CByApiCode;

    @Override
    public void initialize() {
        logger.info(getPluginLabel() + "执行初始化方法initialize()");
    }

    @Override
    public void destroy() {
        logger.info(getPluginLabel() + "执行销毁方法destroy()");
    }

    @Override
    public String getPluginId() {
        return "base.SupplierPluginInitializer";
    }

    @Override
    public String getPluginName() {
        return "丽知：OFS供应商档案--->U8C客商档案";
    }

    @Override
    public String getPluginLabel() {
        return "丽知：OFS供应商档案--->U8C客商档案";
    }

    @Override
    public String getPluginType() {
        return "1";
    }


    @Autowired
    private SaveOrUpdateBusinessLogUtil saveOrUpdateBusinessLogUtil;

    @Autowired
    private IIntegrationTaskLivingDetailsDao iIntegrationTaskLivingDetailsDao;


    @Autowired
    private PushDataByU8cUtil pushDataByU8cUtil;

    @Override
    public JsonResultEntity executeBusiness(JSONObject requestJson) throws Exception {
        try {
            logger.info("调用：" + getPluginName() + "-插件");
            String prod = "prod";

            String param = String.valueOf(requestJson.get("param"));
            if (requestJson != null && ProfilesActiveConstant.TYPE_DATE.equals(requestJson.get("type"))) {
                //按日期
                if (param != null && !"".equals(param)) {
                    String[] split = param.split("/");
                    if (!(split.length == 2)) {
                        Assert.state(false, "时间格式传递不正确");
                    }
                    Assert.notNull(split[0], "开始时间不能为空");
                    Assert.notNull(split[1], "结束时间不能为空");
                    startImplement(split[0], split[1]);
                }
            } else if (requestJson != null && ProfilesActiveConstant.TYPE_VBILLCODE.equals(requestJson.get("type"))) {
                if (param != null && !"".equals(param)) {
                    startImplement(param);
                }
            } else if (ProfilesActiveConstant.TYPE_DETAIL_ERROR.equals(requestJson.get("type"))) {
                String integration_task_living_details_id = (String) requestJson.get("integration_task_living_details_id");
                IntegrationTaskLivingDetailsEntity integrationTaskLivingDetailsEntity = saveOrUpdateBusinessLogUtil.queryIntegrationTaskLivingDetailsEntity(integration_task_living_details_id);
                startImplement(integrationTaskLivingDetailsEntity.getRootAppPk());
            } else if (ProfilesActiveConstant.LETS_PROFILES_ACTIVE.equals(prod)) {
                startImplement();
            }
        } catch (Exception e) {
            logger.error("executeBusiness方法抛出异常", e);
        }
        return BaseResult.getSuccessMessageEntity("插件执行成功");
    }


    /**
     * 默认推送
     */
    public void startImplement() {
        try {
            //获取当前时间
            Date currentDate = new Date();
            Date startTime = DateUtil.offset(currentDate, DateField.MINUTE, -10);
            Date endTime = DateUtil.offset(currentDate, DateField.MINUTE, -2);
            String startTimeStr = DateUtil.format(startTime, "yyyy-MM-dd HH:mm:ss");
            String endTimeStr = DateUtil.format(endTime, "yyyy-MM-dd HH:mm:ss");

            OfsSupplierDto supplierEntity = new OfsSupplierDto();
            supplierEntity.setLastUpdated_start(startTimeStr);
            supplierEntity.setLastUpdated_end(endTimeStr);

            List<OfsSupplierDto> list = queryfromofs(supplierEntity);
            if (list == null || list.size() == 0) {
                return;
            }
            //过滤数据
            List<OfsSupplierDto> supplierDtoList = filterSupplierEntity(list);
            if (supplierDtoList == null || supplierDtoList.size() == 0) {
                return;
            }
            //执行
            implement(supplierDtoList);

        } catch (Exception e) {
            logger.error("丽知：OFS供应商档案--->U8C客商档案,startImplement()方法报错：", e);
        }
    }


    /**
     * 按编码名称
     */
    public void startImplement(String code) {
        String[] split = code.split("_");
        Assert.notNull(code, "code不能为空");
        try {
            OfsSupplierDto supplierEntity = new OfsSupplierDto();
            supplierEntity.setBizCode(split[0]);
            List<OfsSupplierDto> list = queryfromofs(supplierEntity);
            if (list == null || list.size() == 0) {
                Assert.state(false, "根据编码：{}，未查询到ofs供应商档案。", code);
            }
            //过滤数据
            List<OfsSupplierDto> supplierDtoList = filterSupplierEntity(list);
            if (supplierDtoList == null || supplierDtoList.size() == 0) {
                Assert.state(false, "根据编码：{}，已被成功日志过滤。", code);

            }
            //执行
            implement(supplierDtoList);

        } catch (Exception e) {
            logger.error("丽知：OFS供应商档案--->U8C客商档案,startImplement(String code)方法报错：", e);
        }
    }

    /**
     * 按时间区间
     */
    public void startImplement(String startTime, String endTime) {
        try {
            Date business_start = DateUtil.parse(startTime);
            Date business_end = DateUtil.parse(endTime);
            String start = DateUtil.format(business_start, "yyyy-MM-dd") + " 00:00:00";
            String end = DateUtil.format(business_end, "yyyy-MM-dd") + " 23:59:59";
            OfsSupplierDto supplierDto = new OfsSupplierDto();
            supplierDto.setLastUpdated_start(start);
            supplierDto.setLastUpdated_end(end);
            List<OfsSupplierDto> queryfromofs = queryfromofs(supplierDto);
            if (queryfromofs == null || queryfromofs.size() == 0) {
                return;
            }
            //过滤数据
            List<OfsSupplierDto> supplierDtoList = filterSupplierEntity(queryfromofs);
            if (supplierDtoList == null || supplierDtoList.size() == 0) {
                return;
            }
            //执行
            implement(supplierDtoList);
        } catch (Exception e) {
            logger.error("丽知：OFS供应商档案--->U8C客商档案,startImplement(String startTime, String endTime)方法报错：", e);
        }
    }

    /**
     * 过滤日志
     * 编码_最后修改日期
     *
     * @param list
     * @return
     */
    public List<OfsSupplierDto> filterSupplierEntity(List<OfsSupplierDto> list) {

        Assert.notNull(list, "不能为空！");

        List<OfsSupplierDto> supplierDtoList = new ArrayList<>();
        if (list != null && list.size() > 0) {
            for (int i = 0; i < list.size(); i++) {
                OfsSupplierDto ofsSupplierDto = list.get(i);

                if (ofsSupplierDto.getBizCode() != null && !"".equals(ofsSupplierDto.getBizCode().trim())) {
                    boolean isExis = true;
                    IntegrationTaskLivingDetailsEntity integrationTaskLivingDetailsEntity = new IntegrationTaskLivingDetailsEntity();
                    integrationTaskLivingDetailsEntity.setRootAppPk(ofsSupplierDto.getBizCode() + "_" + ofsSupplierDto.getLastUpdated());
                    integrationTaskLivingDetailsEntity.setNewState(ProfilesActiveConstant.LOG_STATUS_Y);
                    integrationTaskLivingDetailsEntity.setPluginId(getPluginId());
                    List<IntegrationTaskLivingDetailsEntity> integrationTaskLivingDetailsEntities = iIntegrationTaskLivingDetailsDao.query(integrationTaskLivingDetailsEntity);
                    if (integrationTaskLivingDetailsEntities == null || integrationTaskLivingDetailsEntities.size() == 0) {
                        isExis = false;
                    }
                    if (!isExis) {
                        supplierDtoList.add(ofsSupplierDto);
                    }
                }
            }
        }
        return supplierDtoList;
    }

    /**
     * 根据名称分组，编码用逗号隔开
     *
     * @param shopEntityList
     */

    @Autowired
    private QueryU8CEntityUtil queryU8CEntityUtil;

    public void implement(List<OfsSupplierDto> shopEntityList) {
        //根据名称分组
        Map<String, List<OfsSupplierDto>> collectByName = shopEntityList.stream().collect(Collectors.groupingBy(index -> index.getName()));
        //根据名称查询u8c客商，有则更新def1，无则新增
        collectByName.forEach((k, v) -> {
            CustdocRequestVo res_custdoc = null;
            String pk_cubasdoc = null;
            String jsonString = null;
            String def1 = null;
            String url = null;
            String custName = k;//客商名称
            //查询u8c客商基本档案
            BdCubasdocEntity bdCubasdocEntity = queryU8CEntityUtil.queryBdCubasdocByName(custName);
            try {
                if (bdCubasdocEntity == null) {
                    //新增
                    url = OverallConstant.getOverAllValue("custdocSaveCode");
                } else {
                    //更新
                    url = OverallConstant.getOverAllValue("custdocUpdate");
                    pk_cubasdoc = bdCubasdocEntity.getPkCubasdoc();
                }

                //拼接def1,如果存在则跳过转换，如果不存在则新增
//                def1 = v.stream().map(OfsSupplierDto::getBizCode).collect(Collectors.joining(","));
                List<String> def1_new = v.stream().map(OfsSupplierDto::getBizCode).collect(Collectors.toList());
                if(bdCubasdocEntity==null){
                    def1 = String.join(",", def1_new);
                }else {
                    String def1_old = bdCubasdocEntity.getDef1();
                    if(def1_old==null){
                        def1 = String.join(",", def1_new);
                    }else {
                        String[] def1_split_old = def1_old.split(",");
                        // 将数组转换为列表
                        List<String> def1_all = new ArrayList<>();
                        List<String> def1_list_old = Arrays.asList(def1_split_old);//u8c里已有的def1
                        def1_all.addAll(def1_list_old);
                        for (String s_new : def1_new) {
                            if (!def1_list_old.contains(s_new)) {
                                def1_all.add(s_new);
                            }
                        }
                        def1 = String.join(",", def1_all);
                    }
                }


                CustdocEntityDto parentvo = new CustdocEntityDto();
                CustdocRequestVo CustdocEntityDto = new CustdocRequestVo();
                if (pk_cubasdoc != null) {
                    parentvo.setPk_cubasdoc(pk_cubasdoc);
                }
                parentvo.setCustname(k);
                parentvo.setCustshortname(k);
                parentvo.setPk_areacl("GG");//20241031:妮妮姐：供应商是BB。202411081502:妮妮姐：供应商是GG
                parentvo.setDef1(def1);

                CustdocEntityDto.setParentvo(parentvo);
                HashMap<String, Object> param = new HashMap<>();
                param.put("cbdocvo", CustdocEntityDto);
                jsonString = JSONObject.toJSONString(param);

                ZTResult ztResult = pushU8CByApiCode.pushByCode(url, jsonString);
                Attribute attribute = ztResult.getAttribute();
                if ("false".equals(ztResult.getFlag())) {
                    //转发失败
                    Assert.state(false, "丽知：OFS供应商档案分类--->U8C客商档案分类.中台转发失败,失败原因:{}", attribute.getErrormsg());
                }
                boolean isSuccess = false;
                if ("success".equals(attribute.getStatus())) {
                    isSuccess = true;
                    String data = attribute.getData();
                    List<CustdocRequestVo> custdocList = JSONUtil.toList(data, CustdocRequestVo.class);
                    res_custdoc = custdocList.get(0);
                }
                if (!isSuccess) {
                    Assert.state(false, "推送U8C，原因：{} 接口入参:{}", attribute.getErrormsg(), jsonString);
                }
                //成功
                // 根据返回结果记录日志
                for (OfsSupplierDto ofsSupplierDto : v) {
                    IntegrationTaskLivingDetailsEntity integrationTaskLivingDetailsEntity = new IntegrationTaskLivingDetailsEntity();
                    integrationTaskLivingDetailsEntity.setNewState(ProfilesActiveConstant.LOG_STATUS_Y);
                    integrationTaskLivingDetailsEntity.setRootAppNewData(jsonString);
                    integrationTaskLivingDetailsEntity.setNewTransmitInfo(attribute.getData());
                    integrationTaskLivingDetailsEntity.setNewPushDate(new Date());
                    integrationTaskLivingDetailsEntity.setBusinessDate(ofsSupplierDto.getLastUpdated());
                    integrationTaskLivingDetailsEntity.setRootAppPk(ofsSupplierDto.getBizCode() + "_" + ofsSupplierDto.getLastUpdated());
                    integrationTaskLivingDetailsEntity.setRootAppBill(ofsSupplierDto.getBizCode());
                    integrationTaskLivingDetailsEntity.setPluginId(getPluginId());
                    if(bdCubasdocEntity!=null){
                        integrationTaskLivingDetailsEntity.setNewSystemNumber(bdCubasdocEntity.getCustcode());
                    }else {
                        integrationTaskLivingDetailsEntity.setNewSystemNumber(res_custdoc.getParentvo().getCustcode());
                    }
                    integrationTaskLivingDetailsEntity.setNewSystemPrimary(res_custdoc.getParentvo().getPk_cubasdoc());
                    saveOrUpdateBusinessLogUtil.saveOrUpdate(integrationTaskLivingDetailsEntity);
                }

            } catch (Exception e) {
                logger.error("ofs供应商档案同步报错：{}", e);
                String ErrMessage = e.getMessage();
                for (OfsSupplierDto ofsSupplierDto : v) {
                    IntegrationTaskLivingDetailsEntity integrationTaskLivingDetailsEntity = new IntegrationTaskLivingDetailsEntity();
                    integrationTaskLivingDetailsEntity.setNewState(ProfilesActiveConstant.LOG_STATUS_N);
                    integrationTaskLivingDetailsEntity.setRootAppNewData(jsonString);
                    integrationTaskLivingDetailsEntity.setNewTransmitInfo(ErrMessage);
                    integrationTaskLivingDetailsEntity.setNewPushDate(new Date());
                    integrationTaskLivingDetailsEntity.setBusinessDate(ofsSupplierDto.getLastUpdated());
                    integrationTaskLivingDetailsEntity.setRootAppPk(ofsSupplierDto.getBizCode() + "_" + ofsSupplierDto.getLastUpdated());
                    integrationTaskLivingDetailsEntity.setRootAppBill(ofsSupplierDto.getBizCode());
                    integrationTaskLivingDetailsEntity.setPluginId(getPluginId());
                    if(bdCubasdocEntity!=null){
                        integrationTaskLivingDetailsEntity.setNewSystemNumber(bdCubasdocEntity.getCustcode());
                    }
                    saveOrUpdateBusinessLogUtil.saveOrUpdate(integrationTaskLivingDetailsEntity);
                }
            }
        });
    }


    public List<OfsSupplierDto> queryfromofs(OfsSupplierDto supplierEntity) throws Exception {
        Long pageNo = 1L;

        List<OfsSupplierDto> all = new ArrayList<>();

        InterfaceParamByU8CApiDto interfaceParamDto = new InterfaceParamByU8CApiDto();
        interfaceParamDto.setApi("ofs.vendor.search");
        supplierEntity.setClientCode("LETS");
        supplierEntity.setPageNo(String.valueOf(pageNo));
        supplierEntity.setPageSize("500");
        interfaceParamDto.setData(JSON.toJSONString(supplierEntity));

        OfsSupplierReturnMessageDto unified = (OfsSupplierReturnMessageDto) ofsUnifiedService.unifiedByU8CApi(interfaceParamDto);
        if (unified.getData() == null || unified.getData().size() == 0) {
            return null;
        }
        all.addAll(unified.getData());
        int count = unified.getData().size();

        while (count >= 500) {//>=500,可能还有下一页。
            //还有下一页
            pageNo++;
            supplierEntity.setPageNo(String.valueOf(pageNo));
            interfaceParamDto.setData(JSON.toJSONString(supplierEntity));
            OfsSupplierReturnMessageDto unified1 = (OfsSupplierReturnMessageDto) ofsUnifiedService.unifiedByU8CApi(interfaceParamDto);
            all.addAll(unified1.getData());
            count = unified1.getData().size();

        }
        return all;
    }
}
