package com.hzya.frame.plugin.lets.util;

import cn.hutool.core.lang.Assert;
import com.hzya.frame.plugin.lets.dao.IIaPeriodaccountDao;
import com.hzya.frame.plugin.lets.dao.IPoOrderBDao;
import com.hzya.frame.plugin.lets.entity.BdInvmandocEntity;
import com.hzya.frame.plugin.lets.entity.IaPeriodaccountEntity;
import com.hzya.frame.plugin.lets.entity.PoOrderBEntity;
import com.hzya.frame.split.SplitListByCountUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 得到存货的结存单价和采购价
 *
 * @Author：liuyang
 * @Package：com.hzya.frame.plugin.lets.util
 * @Project：kangarooDataCenterV3
 * @name：BalanceUnitPriceUtil
 * @Date：2024/10/17 09:17
 * @Filename：BalanceUnitPriceUtil
 */
@Component
public class BalanceUnitPriceUtil {

    @Autowired
    private IIaPeriodaccountDao iIaPeriodaccountDao;

    @Autowired
    private IPoOrderBDao iPoOrderBDao;

    private static final Map<String, String> cacheShopMap = new HashMap<>();

    static {
//        cacheShopMap.put("FANG-Q-L", "FANG-Q-L");
//        cacheShopMap.put("dy-FQL", "dy-FQL");
//        cacheShopMap.put("hzz", "hzz");
//        cacheShopMap.put("pdd-gunanxi", "pdd-gunanxi");
//        cacheShopMap.put("pdd-yanxi", "pdd-yanxi");
//        cacheShopMap.put("pdd-taoxi", "pdd-taoxi");
//        cacheShopMap.put("DLTer", "DLTer");
//        cacheShopMap.put("XZJJ", "XZJJ");

        //202501091118 内部-丽知U8C实施沟通群
        cacheShopMap.put("DLTer", "DLTer");
        cacheShopMap.put("XZJJ", "DLTer");
        cacheShopMap.put("FANG-Q-L-NEW", "DLTer");
        cacheShopMap.put("pdd-yanxi-new", "DLTer");
        cacheShopMap.put("FANG-Q-L", "DLTer");
        cacheShopMap.put("pdd-yanxi", "DLTer");
    }

    /**
     * 判断是否取结存价的店铺
     *
     * @param ofsShop OFS店铺编码
     * @author liuyang
     */
    public Boolean checkOfsShop(String ofsShop) {
        Assert.notNull(ofsShop, "OFS店铺编码不能为空！");
        String cacheValue = cacheShopMap.get(ofsShop);
        if (cacheValue == null) {
            return false;
        }
        return true;
    }

    /**
     * 查询存货的结存金额、数量等信息
     * 需要手动计算一遍，用金额/数量，可能会有负结存的情况，我取绝对值
     *
     * @param bdInvmandocEntity 存货管理档案集合
     * @author liuyang
     */
    public List<IaPeriodaccountEntity> queryBalanceUnitPrice(List<BdInvmandocEntity> bdInvmandocEntity) throws Exception {
//        Assert.notNull(pkCorp, "对应的公司主键不能为空！");

        List<IaPeriodaccountEntity> iaPeriodaccountEntityList = new ArrayList<>();
        if (bdInvmandocEntity != null && bdInvmandocEntity.size() > 0) {
            List<List<BdInvmandocEntity>> lists = SplitListByCountUtil.splitListByCount(bdInvmandocEntity, 600);
            for (int i = 0; i < lists.size(); i++) {
                List<BdInvmandocEntity> bdInvmandocEntities = lists.get(i);

                String pkInvmandocs = bdInvmandocEntities.stream().map(bdInvbasdoc -> "'" + bdInvbasdoc.getPkInvmandoc() + "'").collect(Collectors.joining(","));
                if (pkInvmandocs != null && pkInvmandocs.length() > 0) {
                    IaPeriodaccountEntity iaPeriodaccountEntity = new IaPeriodaccountEntity();
                    iaPeriodaccountEntity.setDataSourceCode("lets_u8c");
//                    iaPeriodaccountEntity.setInvcodes(invcodes);
                    iaPeriodaccountEntity.setPkInvmandocs(pkInvmandocs);
//                    iaPeriodaccountEntity.setPk_corp(pkCorp);
                    List<IaPeriodaccountEntity> iaPeriodaccountEntityList1 = iIaPeriodaccountDao.queryBalancePrice(iaPeriodaccountEntity);
                    iaPeriodaccountEntityList.addAll(iaPeriodaccountEntityList1);
                }
            }
        }
        return iaPeriodaccountEntityList;
    }

    /**
     * 查询存货对应的采购单价
     *
     * @param bdInvmandocEntity 存货管理档案
     * @author liuyang
     */
    public List<PoOrderBEntity> queryPurchaseUnitPriceByInvcodes(List<BdInvmandocEntity> bdInvmandocEntity) throws Exception {
//        Assert.notNull(pkCorp, "对应的公司主键不能为空！");

        List<PoOrderBEntity> poOrderBEntityList = new ArrayList<>();
        if (bdInvmandocEntity != null && bdInvmandocEntity.size() > 0) {
            List<List<BdInvmandocEntity>> lists = SplitListByCountUtil.splitListByCount(bdInvmandocEntity, 600);
            for (int i = 0; i < lists.size(); i++) {
                List<BdInvmandocEntity> bdInvmandocEntities = lists.get(i);

                String cmangids = bdInvmandocEntities.stream().map(bdInvmandocEntity1 -> "'" + bdInvmandocEntity1.getPkInvmandoc() + "'").collect(Collectors.joining(","));
                if (cmangids != null && cmangids.length() > 0) {
                    PoOrderBEntity poOrderBEntity = new PoOrderBEntity();
//                poOrderBEntity.setInvcodes(invcodes);
                    poOrderBEntity.setDataSourceCode("lets_u8c");
                    poOrderBEntity.setCmangids(cmangids);
//                poOrderBEntity.setPk_corp(pkCorp);
                    List<PoOrderBEntity> poOrderBEntityList1 = iPoOrderBDao.queryPurchaseUnitPriceByInvcodes(poOrderBEntity);
                    poOrderBEntityList.addAll(poOrderBEntityList1);
                }
            }
        }
        return poOrderBEntityList;
    }

    /**
     * 初始化「结存价」,每条存货最新的结存价
     * <p>
     * key=存货管理档案
     *
     * @author liuyang
     */
    public Map<String, IaPeriodaccountEntity> initBalancePrice() throws Exception {
        Map<String, IaPeriodaccountEntity> stringIaPeriodaccountEntityMap = new HashMap<>();
        //查询所有存货最新的结存价
        IaPeriodaccountEntity iaPeriodaccountEntity = new IaPeriodaccountEntity();
        iaPeriodaccountEntity.setDataSourceCode("lets_u8c");
        List<IaPeriodaccountEntity> iaPeriodaccountEntityList1 = iIaPeriodaccountDao.queryBalancePrice(iaPeriodaccountEntity);
        if (iaPeriodaccountEntityList1 != null && iaPeriodaccountEntityList1.size() > 0) {
            //转换为Map
            stringIaPeriodaccountEntityMap = iaPeriodaccountEntityList1.stream().filter(entity -> Objects.nonNull(entity.getCinventoryid())).collect(Collectors.toMap(IaPeriodaccountEntity::getCinventoryid, entity -> entity));
        }
        return stringIaPeriodaccountEntityMap;
    }

    /**
     * 初始化「采购价」，每条存货最新的采购价
     * key=存货管理档案
     *
     * @author liuyang
     */
    public Map<String, PoOrderBEntity> initPurchasePrice() throws Exception {
        Map<String, PoOrderBEntity> stringPoOrderBEntityHashMap = new HashMap<>();
        PoOrderBEntity poOrderBEntity = new PoOrderBEntity();
        poOrderBEntity.setDataSourceCode("lets_u8c");
        List<PoOrderBEntity> poOrderBEntityList1 = iPoOrderBDao.queryPurchaseUnitPriceByInvcodes(poOrderBEntity);
        if (poOrderBEntityList1 != null && poOrderBEntityList1.size() > 0) {
            //转换为Map
            stringPoOrderBEntityHashMap = poOrderBEntityList1.stream().filter(entity -> Objects.nonNull(entity.getCmangid())).collect(Collectors.toMap(PoOrderBEntity::getCmangid, entity -> entity));
        }
        return stringPoOrderBEntityHashMap;
    }
}